import java.util.*;
import java.lang.Math;

public class Pole
{
/*
	Some useful Physics:
	
	Moment of intertia through center of rigid rod
	I = 1/12 M L^2
	
	Moment of intertia through end of rigid rod
	I = 1/3 M L^2	
 */
	private Position currPos;
	private double length = 1.0; /* meter*/
	private double vavg = 0.5; /* meters per second */
	private double dt = 0.01; /* second */
	static private double G = 9.8; /* m / s^2 */
	
	public Pole(Position p, double l)
	// Constructor
	{
		this.currPos = p;
		this.length = l;
	}
	
	private int checkPos(Position p)
		/*
		returns -1 : fell left
		returns 0 : remained upright
		returns 1 : fell right
		 */
	{
		double pi = 3.14159;
		if (p.theta > (pi / 2))
			return 1;
		else if (p.theta < (-pi / 2))
			return -1;
		else
			return 0;
	}
	
	private int iter(double ax, Position currPos, Position newPos/*, double ay*/)
	/*
		Executes each of the ten little iterations.
		
		ax is the acceleration of the base in the x - direction
		
		newPos is the new position that is generated
		
		returns -1 : fell left
		returns 0 : remained upright
		returns 1 : fell right
	*/
	{
		double theta = currPos.theta;
		
		double alpha1, alpha2, alphatot; /*angular acceleration values*/

		/* Effects of gravity */
		alpha1 = (3 * G * Math.sin(theta) ) / (2 * length);
		
		/* Effects of force on end of pole */
		alpha2 = - ( 2 * ax * Math.cos(theta) ) / length;
		
		alphatot = alpha1 + alpha2;
		
		/* Iterate one step forward in time */
		newPos.theta_dot = currPos.theta_dot + alphatot * dt;
		newPos.xbase_dot = currPos.xbase_dot + ax * dt;
		newPos.theta = currPos.theta + newPos.theta_dot * dt;
		newPos.xbase = currPos.xbase + newPos.xbase_dot * dt;
		
		return checkPos(newPos);
	}
	
	public int step(int xmotion, Vector tenMoves/*, int ymotion*/)
	/*	 xmotion can be -1, 0, or 1 depending on
		 whether in a given iteration, the base moves
		 to the left, stays stationary, or moves to the
		 right.  The same applies to ymotion.

		The motion consists of ten steps.  When there is motion,
		it consists of constant acceleration (of the base) for the 
		first five time steps and constant deceleration (of the base)
		for the next five time steps.
		
		Each step consists of ten iterations, which execute the physics
		of each small step.  If the pole fell during the process, the remainder
		of the tenMoves vector is filled with the last good move.
		
		returns -1 : fell left
		returns 0 : remained upright
		returns 1 : fell right
		
		tenMoves is a vector which contains information on each of the 
		ten iteration steps for the sake of display.
	*/	
	{
		Position newPos;
		int i;
		double ax = 0.0; /*linear acceleration in the x - direction */
		int fell;
		tenMoves.removeAllElements();
		
		/* do positive acceleration portion */
		ax = vavg * xmotion * 0.4 / dt;
		for (i = 0; i < 5; i ++)
		{
			newPos = new Position();
			fell =  iter(ax,currPos, newPos);
			tenMoves.addElement(newPos);
			currPos = newPos;
			if (fell != 0)
			{
				Fill(tenMoves);
	//			System.out.println("Tenmoves - size = "+tenMoves.size());			
				return fell;
			}
		}
		
		/* do negative acceleration portion */
		ax = - vavg * xmotion * 0.4 / dt; 		
		for (i = 0; i < 5; i ++)
		{
			newPos = new Position();
			fell =  iter(ax,currPos, newPos);
			tenMoves.addElement(newPos);
			currPos = newPos;
			if (fell != 0)
			{
				Fill(tenMoves);
//			System.out.println("Tenmoves sjdh size = "+tenMoves.size());			
				return fell;
			}
		}	
		
		return 0;
	}
	
	public void Fill(Vector tenmoves)
	{
//		System.out.println("FILLING!!");
		Position p = (Position)tenmoves.elementAt(tenmoves.size() - 1);
		for(int i = tenmoves.size(); i < 10; i++)
			tenmoves.addElement(p);
//		System.out.println("Tenmoves size = "+tenmoves.size());
	}
	
	public Position getPos()
	{
		return this.currPos;
	}
	
	public void setPos(Position p)
	{
		this.currPos = p;
	}
}